-- =============================================================================
-- Copyright © 2017-2018 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Farnoud Farahmand
-- =============================================================================

library ieee;
use ieee.std_logic_1164.all;

entity BPGM_MGF_Wrapper is
    generic (
        W             : integer := 32
    );
    port (
        --! Global signals
        clk             : in  std_logic;
        rst             : in  std_logic;

        --! SERDES signals
        sin             : in  std_logic;
        ssel            : in  std_logic;
        sout            : out std_logic
    );
end entity BPGM_MGF_Wrapper;

architecture structure of BPGM_MGF_Wrapper is
    signal sipo         : std_logic_vector((5*W)+256+6+8   -1 downto 0);
    signal piso         : std_logic_vector(6+3+3+11+128+7  -1 downto 0);
    signal piso_data    : std_logic_vector(6+3+3+11+128+7  -1 downto 0);
begin
    process(clk)
    begin
        if rising_edge(clk) then
            sipo <= sin & sipo(sipo'high downto 1);
            if (ssel = '1') then
                piso <= piso_data;
            else
                piso <= '0' & piso(piso'high downto 1);
            end if;
        end if;
    end process;
    sout <= piso(0);

    u_bpgm_mgf:
    entity work.bpgm_mgf(arch)
    port map (
        clk                     => clk                          ,
        rst                     => rst                          ,

        --! Input signals
        pdi_data                => sipo(            W-1 downto  0),
        b                       => sipo(        (2*W)-1 downto  W),
        htrunc                  => sipo(        (3*W)-1 downto (2*W)),
        cm                      => sipo(        (4*W)-1 downto (3*W)),
        cb                      => sipo(        (5*W)-1 downto (4*W)),
        R4                      => sipo(    128+(5*W)-1 downto (5*W)),
        cR4                     => sipo(    256+(5*W)-1 downto 128+(5*W)),
        msg_len                 => sipo(  8+256+(5*W)-1 downto 256+(5*W)),
        bpgm_fifo_read          => sipo(  8+256+(5*W)),
        mask_ready              => sipo(1+8+256+(5*W)),
        start                   => sipo(2+8+256+(5*W)),
        decrypt                 => sipo(3+8+256+(5*W)),
        sel_param               => sipo(4+8+256+(5*W)),
        sel_bpgm_mgf            => sipo(5+8+256+(5*W)),
        --! Output signals
        pdi_addr                => piso_data(             6-1 downto 0),
        b_addr                  => piso_data(           3+6-1 downto 6),
        h_addr                  => piso_data(         3+3+6-1 downto 3+6),
        bpgm_fifo_dout          => piso_data(      11+3+3+6-1 downto 3+3+6),
        mask_data               => piso_data(  128+11+3+3+6-1 downto 11+3+3+6),
        pdi_rd                  => piso_data(  128+11+3+3+6),
        b_rd                    => piso_data(1+128+11+3+3+6),
        h_rd                    => piso_data(2+128+11+3+3+6),
        bpgm_fifo_empty         => piso_data(3+128+11+3+3+6),
        bpgm_fifo_almost_full   => piso_data(4+128+11+3+3+6),
        mask_valid              => piso_data(5+128+11+3+3+6),
        done                    => piso_data(6+128+11+3+3+6)
    );
end structure;
