--=========================================================
-- Project      : Hardware Implementation of NTRU Cryptosystems
-- Author       : Kevin Briggs
--=========================================================
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

package func_pkg is
    function has_rmndr(X : integer) return integer;
    function divceil(a  : natural; b : natural) return natural;
    function log2ceil(num : natural) return natural;
    function slv_to_string( v : std_logic_vector) return string;
end func_pkg;



-- functions description
package body func_pkg is

    function has_rmndr(X : integer) return integer is
    begin
        if(X > 0) then
            return 1;
        else
            return 0;
        end if;
    end function;

    function divceil (a : natural; b : natural ) return natural is
        begin
            if(a mod b) > 0 then
                return (a / b) + 1;
            else
                return (a / b);
            end if;
        end function;

    function log2ceil ( num : natural) return natural is
            variable i    : natural;
            variable bits : natural;
        begin
            i := 0;
            while(2**i < num) loop
                i := i + 1;
            end loop;
            return i;
        end function;

    function slv_to_string( v : std_logic_vector) return string is
                variable s : string (v'length-1 downto 0) := (others => NUL);
            begin
                for i in v'length - 1 downto 0 loop
                    s(i) := std_logic'image(v(i))(2);
                end loop;
                return s;
            end function;
end package body func_pkg;

-- ==========================================================================
