--=========================================================
-- Project 	: Hardware Implementation of NTRU Cryptosystems
-- Author	: Kevin Briggs, Malik Umar Sharif 
--=========================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all; 

library work;
use work.ntru_pkg.all;

-- Input Data Conversion Unit for encryption
entity idcu_e is	
generic 
(
	iw 	     : integer := 32;
	octl_len : integer := 8
);
port
(   
	-- inputs
	clk 	 : in std_logic;
	rst 	 : in std_logic;
	pdi_data : in std_logic_vector(iw-1 downto 0);
	rdi_data : in std_logic_vector(iw-1 downto 0);
	
    en_reg_octl : in std_logic;
	
	-- outputs
	octl_sig    : out std_logic_vector(octl_len-1 downto 0);
	m_in        : out std_logic_vector(iw-1 downto 0);
	b_in        : out std_logic_vector(iw-1 downto 0)
); 
end idcu_e;

architecture arch of idcu_e is
    constant zero : std_logic_vector(octl_len - 1 downto 0) := (others => '0');
begin

    octl_reg : entity work.regn generic map ( N =>octl_len, init => zero)
    port map ( clk => clk, rst => rst, en => en_reg_octl, input =>pdi_data(iw-1 downto iw-8), output =>octl_sig);
    
    m_in <= pdi_data;
    b_in <= rdi_data;

end arch;