--=========================================================
-- Project 	: Hardware Implementation of NTRU Cryptosystems
-- Author	: Malik Umar Sharif
--=========================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.ntru_pkg.all;
use work.sha3_pkg.all;
use work.sha2_pkg.all;

entity poly_add is
	generic
	(
		w	: integer := 32;
		ADDR_WIDTH : integer := 4;
		DATA_WIDTH : integer := 2;
        ALPHA : integer := 11
	);
	port
	(
		a   : in std_logic_vector(2*2*w-1 downto 0);
		b   : in std_logic_vector(2*2*w-1 downto 0);
		c 	: in std_logic_vector(2*w*ALPHA-1 downto 0);
        m_pr: out std_logic_vector(2*2*w-1 downto 0);
		r   : out std_logic_vector(2*w*ALPHA-1 downto 0)
	);
end poly_add;

architecture arch of poly_add is

	type rom_modp_array is array (0 to 2**ADDR_WIDTH-1) of std_logic_vector(DATA_WIDTH-1 downto 0);
	constant rom_modp : rom_modp_array := ("00", "01", "10", "00", "01", "10", "00", "01", "10", "00", "01", "10", "00", "01", "10", "00" );

	type abcr_array is array (0 to 2*w-1) of std_logic_vector(1 downto 0);
	signal ai, bi, rom_out : abcr_array;

	type addr_array is array (0 to 2*w-1) of std_logic_vector(3 downto 0);
	signal addr : addr_array;

	type cr_array is array (0 to 2*w-1) of std_logic_vector(ALPHA-1 downto 0);
	signal ci, ri : cr_array;

	signal tt : std_logic_vector(ALPHA-1 downto 0);

begin
	ii : for i in 0 to 2*w-1 generate
		ai(i) <= a(2*(i+1)-1 downto 2*i);
		bi(i) <= b(2*(i+1)-1 downto 2*i);
		ci(i) <= c(ALPHA*(i+1)-1 downto ALPHA*i);
		addr(i) <= ai(i) & bi(i);
		rom_out(i) <= rom_modp(to_integer(unsigned (addr(i))));

		with rom_out(i) select
            ri(i) <= 	std_logic_vector(unsigned (ci(i)) - ('0' & '1')) when "10",
                        std_logic_vector(unsigned (ci(i)) + ('0' & '1')) when "01",
                        ci(i) when others;

        m_pr(2*(i+1)-1 downto 2*i) <= rom_out(i);
		r(ALPHA*(i+1)-1 downto ALPHA*i) <= ri(i);
	end generate;
end arch;

-- =============================================================
