--=========================================================
-- Project 	: Hardware Implementation of NTRU Cryptosystems
-- Author	: Malik Umar Sharif
--=========================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.ntru_pkg.all;
use work.sha3_pkg.all;
use work.sha2_pkg.all;

entity poly_sub_2 is
	generic
	(
		w	: integer := 32;
		ADDR_WIDTH : integer := 4;
		DATA_WIDTH : integer := 2;
        ALPHA   : integer := 11
	);
	port
	(
		a   : in std_logic_vector(2*w*ALPHA-1 downto 0);
		b   : in std_logic_vector(2*2*w-1 downto 0);
		r   : out std_logic_vector(2*w*ALPHA-1 downto 0)
	);
end poly_sub_2;

architecture arch of poly_sub_2 is

	type ai_array is array (0 to 2*w-1) of std_logic_vector(ALPHA-1 downto 0);
	signal ai : ai_array;

	type bi_array is array (0 to 2*w-1) of std_logic_vector(1 downto 0);
	signal bi : bi_array;

	type r_array is array (0 to 2*w-1) of std_logic_vector(ALPHA-1 downto 0);
	signal ri : r_array;

begin
	ii : for i in 0 to 2*w-1 generate
		ai(i) <= a(ALPHA*(i+1)-1 downto ALPHA*i);
		bi(i) <= b(2*(i+1)-1 downto 2*i);

		with bi(i) select
			ri(i) <=
                        std_logic_vector(unsigned (ai(i)) - ('0' & '1')) when "01",
                        std_logic_vector(unsigned (ai(i)) + ('0' & '1')) when "10",
						ai(i) when others;

		r(ALPHA*(i+1)-1 downto ALPHA*i) <= ri(i);
	end generate;
end arch;

-- =============================================================
