-- =====================================================================
-- Copyright © 2010-2012 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use work.sha3_pkg.all;
use work.sha2_pkg.all;

entity sha2_flags_bl is
generic (
	n 				:integer :=HASH_SIZE_256/SHA2_WORDS_NUM;    	-- size of basic operations
	s 				:integer:=LOG_2_8; 		-- width of ctr for number of blocks of hash digest
	flag			:integer:=HASH_BLOCKS_256; -- number of blocks of hash digest
	a 				:integer:=LOG_2_64;  	-- width of address bus
	r				:integer:=ROUNDS_64);	--number of rounds
port (
	clk					:in 	std_logic;
	en					:in std_logic;
	wr_len				:in 	std_logic;
	last_block_in		:in 	std_logic;
	last_block_out		:out 	std_logic;
	rd_num				:in 	std_logic_vector(a-1 downto 0);
	exam_block			:in 	std_logic_vector(n-1 downto 0);
	out_ctr				:in		std_logic_vector(s downto 0);
	rst					:in 	std_logic;
	rst_flags			:in 	std_logic;
	z16					:out 	std_logic;
	zlast				:out 	std_logic;
	o8					:out	std_logic);
end sha2_flags_bl;

architecture sha2_flags_bl of sha2_flags_bl is


signal z16_in			:std_logic;
signal z16_out			:std_logic;
signal zlast_in			:std_logic;
signal zlast_out		:std_logic;
signal o8_in			:std_logic;

signal zero				:std_logic_vector(n-1 downto 0);
constant cmp				:std_logic_vector(s downto 0) := std_logic_vector(to_unsigned(flag, s+1));


begin

	zero <= (others=>'0');

	z16_in <= '1' when (rd_num=std_logic_vector(to_unsigned(ROUND_16-5, a))) else '0';
	zlast_in <= '1' when (rd_num=std_logic_vector(to_unsigned(r-5, a))) else '0';

	o8_in <= '1' when (out_ctr=cmp) else '0';

	r0: entity work.d_ff(d_ff) port map (clk=>clk, rst=>rst, ena=>en, d=>z16_in, q=>z16_out);
	r1: entity work.d_ff(d_ff) port map (clk=>clk, rst=>rst, ena=>en, d=>zlast_in, q=>zlast_out);
   	r4: entity work.d_ff(d_ff) port map (clk=>clk, rst=>rst_flags, ena=>wr_len, d=>last_block_in, q=>last_block_out); 

	z16 <= z16_out;
	zlast <= zlast_out;
	o8 <= o8_in;

end sha2_flags_bl;
