-- =========================================================================
-- Copyright © 2017-2018 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- Author: Farnoud Farahmand
-- =========================================================================
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
library work;
use work.ntru_pkg.all;

entity top_datapath is
    port(
        clk       : in  std_logic;
        rst       : in  std_logic;

        sdi_data        : in std_logic_vector(15 downto 0);
        pdi_data        : in std_logic_vector(31 downto 0);
        rdi_data        : in std_logic_vector(31 downto 0);
        do_data         : out std_logic_vector(31 downto 0);

        ctrl_dout       : in std_logic_vector(31 downto 0);
        sel_ctrl_header : in std_logic;
        ctrl_do_valid   : in std_logic;
        ctrl_do_ready   : out std_logic;

        R_fe_valid_to_ctrl : out std_logic;
        Rfe_rd_from_ctrl   : in std_logic;
        sel_Rfe_rd         : in std_logic;

        he_fifo_read    : out std_logic;
        he_fifo_empty   : in std_logic;

        mask_ready      : in std_logic;
        mask_valid      : out std_logic;

        ce_valid        : out std_logic;
        ce_read         : in std_logic;

        bpgm_mgf_start  : in std_logic;
        bpgm_mgf_done   : out std_logic;
        bpgm_mgf_sel    : in std_logic;
        bpgm_mgf_rst    : in std_logic;
        chk_failed_bwc1 : out std_logic;
        chk_failed_bwc2 : out std_logic;
        msg_len         : in std_logic_vector(7 downto 0);
        bpgm_mgf_Rferd  : out std_logic;

        decrypt         : in std_logic;
        sel_param       : in std_logic;
        check_3         : out std_logic;

        check1_rst      : in std_logic;
        check1_en_reg   : in std_logic;
        check1_last     : in std_logic;
        check_1         : out std_logic;

        poly_mult_rst   : in std_logic;
        poly_mult_update_h  : in std_logic;
        poly_mult_sdi_ready : in std_logic;
        poly_mult_start : in std_logic;
        poly_mult_done  : out std_logic;
        poly_mult_rF_sel: in std_logic;


        sel_r_F         : in std_logic;
        sel_m_cm_di     : in std_logic;
        sel_b_cb_di     : in std_logic;
        sel_hTrunc_addr : in std_logic;
        sel_odcu        : in std_logic;
        sel_ci_mpr      : in std_logic;

        sel_m_cm_addr   : in std_logic;
        sel_b_cb_addr   : in std_logic;


        --! hTrunc Distributed RAM ports
        hTrunc_we       : in std_logic;
        hTrunc_addr_ctrl: in std_logic_vector(2 downto 0);
        --! F Distributed RAM ports
        sel_F_addr      : in std_logic;
        F_we            : in std_logic;
        F_addr_ctrl     : in std_logic_vector(7 downto 0);
        --! m_cm  BRAM ports
        m_cm_we         : in std_logic;
        m_cm_rd_ctrl    : in std_logic;
        m_cm_addr_ctrl  : in std_logic_vector(5 downto 0);
        --! b_cb Distributed RAM ports
        b_cb_we         : in std_logic;
        b_cb_addr_ctrl  : in std_logic_vector(2 downto 0);
        --! DFU-M controling signal
        dfum_sel_regoct : in std_logic;
        dfum_sel_data   : in std_logic;
        dfum_sel_sipo   : in std_logic;
        dfum_en_mreg    : in std_logic;
        dfum_en_sipo    : in std_logic;
        --! ODCU-E controling signal
        odcue_di_valid  : in std_logic;
        odcue_di_ready  : out std_logic;
        --! ODCU-D controling signal
        odcud_en_piso   : in std_logic;
        odcud_ld_piso   : in std_logic;
        odcud_en_mreg   : in std_logic;
        odcud_sel_cm    : in std_logic;
        odcud_sel_cb    : in std_logic;
        odcud_coctL     : out std_logic_vector(7 downto 0);
        odcud_check2    : out std_logic_vector(3 downto 0);
        --! IDCU-ED controling signal
        idcued_en_sipo_he   : in std_logic;
        idcued_en_sipo_hram : in std_logic;
        idcued_en_piso_hram : in std_logic;
        idcued_ld_piso_hram : in std_logic;
        --! do bus
        do_valid        : out std_logic;
        do_ready        : in std_logic
    );



end top_datapath;

architecture structure of top_datapath is

signal m_cm_addr_bpg, m_cm_addr
    : std_logic_vector(5 downto 0);

signal odcud_dout, odcue_dout
    : std_logic_vector(31 downto 0); -- WE SHOULD change it to 64 in the future
signal hTrunc_addr, b_cb_addr, b_cb_addr_bpg, hTrunc_addr_bpg, hTrunc_addr_idc
    : std_logic_vector(2 downto 0);
signal poly_mult_F_addr, F_addr
    : std_logic_vector(7 downto 0);

signal Rfe_rd_from_bpgmmgf, R_fe_valid, R_fe_read, m_cm_rd, i_fifo_empty,
    i_fifo_read, bpgm_fifo_almost_full, m_cm_rd_bpg, odcue_do_valid,
    odcue_do_ready, check_1_s, odcud_do_valid, odcud_do_ready
    : std_logic;
signal m_cm_di, b_cb_di, m_cm_do, b_cb_do, hTrunc_do, idcued_do, cm, cb,
    idcue_m, idcue_b
    : std_logic_vector(31 downto 0);

signal R4, cR4, b2t_do, mask_data, ci, cMtrin, check1_di, m_pr
    : std_logic_vector(127 downto 0);
signal dfum_do, cMbin
    : std_logic_vector(95 downto 0);
signal R_fe, e, ce, cR
    : std_logic_vector(703 downto 0);

signal F_di, F_do, r_F, i_fifo_dout
    : std_logic_vector(10 downto 0);

signal h_e
    : std_logic_vector(21 downto 0);

--DEBUG
type debug_array is array (0 to 63) of std_logic_vector(10 downto 0);
signal R_fe_debug : debug_array;
--DEBUG


begin

    IDCU_D: entity work.idcu_d(arch)
        generic map(
            iw => 16,
            ow => 11
        )
        port map(
            sdi_data => sdi_data,
            F_ram    => F_di
        );

    --! Distributed RAM
    F_RAM: entity work.rams_dist(syn)
        generic map(
        W => 11,
        addr_W => 8,
        addr_S => 256
        )
        port map(
            clk   => clk,
            we    => F_we,
            a     => F_addr,
            di    => F_di,
            do    => F_do
        );

    r_F <= F_do when sel_r_F = '1' else i_fifo_dout;
    F_addr <= F_addr_ctrl when sel_F_addr = '1' else poly_mult_F_addr;

    poly_mult_top : entity work.poly_mult_top (arch)
    generic map (
        N => PARAM_N,
        ALPHA => 11,
        BETA => 11,
        W    => 32,
        PIPE_N => PIPE_N,
        PIPE_STAGES => PIPE_STAGES
    )
    port map (
       clk           => clk,
       rst_dp        => rst,
       rst_ctrl      => poly_mult_rst,

       r_F_sel       => poly_mult_rF_sel,
       sdi_addr      => poly_mult_F_addr,
       sdi_ready     => poly_mult_sdi_ready,

       param         => '0', --t_param
       update_h      => poly_mult_update_h,
       decrypt       => decrypt,
       start         => poly_mult_start,
       done          => poly_mult_done,

       ce_valid      => ce_valid,
       ce_read       => ce_read,
       ce            => ce,

       he_fifo_empty => he_fifo_empty,
       he_fifo_read  => he_fifo_read,
       h_e           => h_e,

       i_fifo_empty  => i_fifo_empty,
       i_fifo_read   => i_fifo_read,
       r_F           => r_F,

       R_fe          => R_fe,
       Rfe_valid     => R_fe_valid,
       Rfe_read      => R_fe_read,
       cR            => cR
   );

   R_fe_valid_to_ctrl <= R_fe_valid;
   R_fe_read <= Rfe_rd_from_ctrl when sel_Rfe_rd = '1' else Rfe_rd_from_bpgmmgf;
   bpgm_mgf_Rferd <= Rfe_rd_from_bpgmmgf;

    bpgm_mgf_inst : entity work.bpgm_mgf
    port map(
        clk        => clk,
        rst        => bpgm_mgf_rst,

        pdi_data   => m_cm_do,
        pdi_addr   => m_cm_addr_bpg,
        pdi_rd     => m_cm_rd_bpg,

        b          => b_cb_do,
        b_addr     => b_cb_addr_bpg,
        b_rd       => open,

        htrunc     => hTrunc_do,
        h_addr     => hTrunc_addr_bpg,
        h_rd       => open,

        R4         => R4,
        cR4        => cR4,
        R_fe_valid => R_fe_valid,
        R_fe_read  => Rfe_rd_from_bpgmmgf,

        bpgm_fifo_dout  => i_fifo_dout,
        bpgm_fifo_empty => i_fifo_empty,
        bpgm_fifo_read  => i_fifo_read,
        bpgm_fifo_almost_full => bpgm_fifo_almost_full,

        mask_data   => mask_data,
        mask_valid  => mask_valid,
        mask_ready  => mask_ready,

		start         => bpgm_mgf_start,
        chk_failed_bwc1 => chk_failed_bwc1,
        chk_failed_bwc2 => chk_failed_bwc2,
        done          => bpgm_mgf_done,
		decrypt       => decrypt,
        sel_param     => sel_param,
        sel_bpgm_mgf  => bpgm_mgf_sel,
        msg_len       => msg_len
	);

m_cm_di   <= cm when sel_m_cm_di = '1' else idcue_m;
m_cm_addr <= m_cm_addr_ctrl when sel_m_cm_addr = '1' else m_cm_addr_bpg;
m_cm_rd   <= m_cm_rd_ctrl when sel_m_cm_addr = '1' else m_cm_rd_bpg;

--! BRAM
m_cm_RAM: entity work.rams_sp_rf_rst(syn)
    generic map(
    W => 32,
    addr_W => 6,
    addr_S => 64
    )
    port map(
        clk   => clk,
        en    => m_cm_rd,
        we    => m_cm_we,
        rst   => rst,
        addr  => m_cm_addr,
        di    => m_cm_di,
        do    => m_cm_do
    );

b_cb_di   <= cb when sel_b_cb_di = '1' else idcue_b;
b_cb_addr <= b_cb_addr_ctrl when sel_b_cb_addr = '1' else b_cb_addr_bpg;

--! Distributed RAM
b_cb_RAM: entity work.rams_dist(syn)
    generic map(
    W => 32,
    addr_W => 3,
    addr_S => 8
    )
    port map(
        clk   => clk,
        we    => b_cb_we,
        a     => b_cb_addr,
        di    => b_cb_di,
        do    => b_cb_do
    );

IDCU_E: entity work.idcu_e(arch)
    generic map(
        iw => 32
    )
    port map(
        clk      => clk,
        rst      => rst,
        pdi_data => pdi_data,
        rdi_data => rdi_data,
        en_reg_octl => '0',

        octl_sig => open,
        m_in     => idcue_m,
        b_in     => idcue_b
    );

IDCU_ED: entity work.idcu_ed(arch)
    generic map(
        iw       => 32,
        ow_h_e   => 22,
        ow_h_ram => 32
    )
    port map(
        clk      => clk,
        rst      => rst,

        en_sipo_h_e   => idcued_en_sipo_he,
        en_sipo_h_ram => idcued_en_sipo_hram,
        en_piso_h_ram => idcued_en_piso_hram,
        ld_piso_h_ram => idcued_ld_piso_hram,

        pdi_data => pdi_data,
        h_e      => h_e,
        h_ram    => idcued_do
    );

--! Distributed RAM
hTrunc_RAM: entity work.rams_dist(syn)
    generic map(
    W => 32,
    addr_W => 3,
    addr_S => 8
    )
    port map(
        clk   => clk,
        we    => hTrunc_we,
        a     => hTrunc_addr,
        di    => idcued_do,
        do    => hTrunc_do
    );

hTrunc_addr <= hTrunc_addr_ctrl when sel_hTrunc_addr = '1' else hTrunc_addr_bpg;



R4_mod4: entity work.mod4(arch)
    port map(
        din  => R_fe,
        dout => R4
    );

cR4_mod4: entity work.mod4(arch)
    port map(
        din  => cR,
        dout => cR4
    );

DFU_M: entity work.dfu_m(behavioral)
    generic map(
        iw => 32
    )
    port map(
        clk => clk,
        rst => rst,

        sel_reg_oct  => dfum_sel_regoct,
        sel_data_sig => dfum_sel_data,
        sel_sipo_in  => dfum_sel_sipo,
        en_mreg      => dfum_en_mreg,
        en_sipo      => dfum_en_sipo,

        m            => m_cm_do,
        b            => b_cb_do,
        octl         => msg_len,

        Mbin         => dfum_do
    );

b2t_ints: entity work.b2t(arch)
    port map(
        input  => dfum_do,
        output => b2t_do
    );

poly_add_inst: entity work.poly_add(arch)
    port map(
        a => mask_data,
        b => b2t_do,
        c => R_fe,
        m_pr => m_pr,
        r => e
    );

ODCU_E: entity work.odcu_e(arch)
    generic map(
        ow => 32
    )
    port map(
        clk     => clk,
        rst     => rst,

        check_1  => check_1_s,

        di_valid => odcue_di_valid,
        di_ready => odcue_di_ready,
        di       => e,

        do_valid => odcue_do_valid,
        do_ready => odcue_do_ready,
        do_data  => odcue_dout
    );

do_data  <= odcud_dout when sel_odcu = '1' else odcue_dout;
do_valid <= ctrl_do_valid when sel_odcu = '1' else odcue_do_valid;
odcue_do_ready <= '0' when sel_odcu = '1' else do_ready;
ctrl_do_ready <= do_ready when sel_odcu = '1' else '0';

poly_sub_1_inst: entity work.poly_sub_1(arch)
    port map(
        a => ci, --mask_data
        b => mask_data, --ci
        r => cMtrin
    );

t2b_ints: entity work.t2b(arch)
    port map(
        a  => cMtrin,
        r  => cMbin
    );

--! add odcud_do_valid and odcud_do_ready
ODCU_D_Check2: entity work.odcu_d(arch)
    generic map(
        ow => 32
    )
    port map(
        clk     => clk,
        rst     => rst,

        en_piso => odcud_en_piso,
        ld_piso => odcud_ld_piso,
        en_mreg => odcud_en_mreg,
        sel_cm  => odcud_sel_cm,
        sel_cb  => odcud_sel_cb,

        cMbin   => cMbin,

        cm      => cm,
        cb      => cb,
        coctL   => odcud_coctL,
        check_2 => odcud_check2
    );
    -- odcud_dout
    odcud_dout <= ctrl_dout when sel_ctrl_header = '1' else m_cm_do;

Range_conv_modp: entity work.range_conv_mod_p(arch)
    port map(
        rc_full => R_fe,
        ci_full => ci
    );

check1_di <= ci when decrypt = '1' else m_pr;

--check_1:
Check_1_inst: entity work.check_1(arch)
    port map(
        clk     => clk,
        rst     => check1_rst,
        check1_last   => check1_last,
        en_check1_reg => check1_en_reg,
        mp_ci   => check1_di,
        check_1 => check_1_s
    );

check_1 <= check_1_s;

poly_sub_2_inst: entity work.poly_sub_2(arch)
    port map(
        a => ce,
        b => ci,
        r => cR
    );

--! Check 3
check_3 <= '1' when (R_fe = ce) else '0';

--DEBUG
-- GEN_DEBUG0: for i in 0 to 63 generate
--     R_fe_debug(i) <= R_fe(11*(i+1)-1 downto 11*i);
-- end generate;
--DEBUG

end structure;
